/* gamepad-button-type.c generated by valac 0.36.4, the Vala compiler
 * generated from gamepad-button-type.vala, do not modify */

/* This file is part of retro-gtk. License: GPLv3*/

#include <glib.h>
#include <glib-object.h>


#define RETRO_TYPE_GAMEPAD_BUTTON_TYPE (retro_gamepad_button_type_get_type ())

typedef enum  {
	RETRO_GAMEPAD_BUTTON_TYPE_ACTION_DOWN,
	RETRO_GAMEPAD_BUTTON_TYPE_ACTION_LEFT,
	RETRO_GAMEPAD_BUTTON_TYPE_SELECT,
	RETRO_GAMEPAD_BUTTON_TYPE_START,
	RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_UP,
	RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_DOWN,
	RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_LEFT,
	RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_RIGHT,
	RETRO_GAMEPAD_BUTTON_TYPE_ACTION_RIGHT,
	RETRO_GAMEPAD_BUTTON_TYPE_ACTION_UP,
	RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_L,
	RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_R,
	RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_L2,
	RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_R2,
	RETRO_GAMEPAD_BUTTON_TYPE_STICK_L,
	RETRO_GAMEPAD_BUTTON_TYPE_STICK_R,
	RETRO_GAMEPAD_BUTTON_TYPE_HOME
} RetroGamepadButtonType;



gsize retro_gamepad_button_type_size (void);
GType retro_gamepad_button_type_get_type (void) G_GNUC_CONST;


gsize retro_gamepad_button_type_size (void) {
	gsize result = 0UL;
	result = 1 + ((gsize) RETRO_GAMEPAD_BUTTON_TYPE_HOME);
	return result;
}


GType retro_gamepad_button_type_get_type (void) {
	static volatile gsize retro_gamepad_button_type_type_id__volatile = 0;
	if (g_once_init_enter (&retro_gamepad_button_type_type_id__volatile)) {
		static const GEnumValue values[] = {{RETRO_GAMEPAD_BUTTON_TYPE_ACTION_DOWN, "RETRO_GAMEPAD_BUTTON_TYPE_ACTION_DOWN", "action-down"}, {RETRO_GAMEPAD_BUTTON_TYPE_ACTION_LEFT, "RETRO_GAMEPAD_BUTTON_TYPE_ACTION_LEFT", "action-left"}, {RETRO_GAMEPAD_BUTTON_TYPE_SELECT, "RETRO_GAMEPAD_BUTTON_TYPE_SELECT", "select"}, {RETRO_GAMEPAD_BUTTON_TYPE_START, "RETRO_GAMEPAD_BUTTON_TYPE_START", "start"}, {RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_UP, "RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_UP", "direction-up"}, {RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_DOWN, "RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_DOWN", "direction-down"}, {RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_LEFT, "RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_LEFT", "direction-left"}, {RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_RIGHT, "RETRO_GAMEPAD_BUTTON_TYPE_DIRECTION_RIGHT", "direction-right"}, {RETRO_GAMEPAD_BUTTON_TYPE_ACTION_RIGHT, "RETRO_GAMEPAD_BUTTON_TYPE_ACTION_RIGHT", "action-right"}, {RETRO_GAMEPAD_BUTTON_TYPE_ACTION_UP, "RETRO_GAMEPAD_BUTTON_TYPE_ACTION_UP", "action-up"}, {RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_L, "RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_L", "shoulder-l"}, {RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_R, "RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_R", "shoulder-r"}, {RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_L2, "RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_L2", "shoulder-l2"}, {RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_R2, "RETRO_GAMEPAD_BUTTON_TYPE_SHOULDER_R2", "shoulder-r2"}, {RETRO_GAMEPAD_BUTTON_TYPE_STICK_L, "RETRO_GAMEPAD_BUTTON_TYPE_STICK_L", "stick-l"}, {RETRO_GAMEPAD_BUTTON_TYPE_STICK_R, "RETRO_GAMEPAD_BUTTON_TYPE_STICK_R", "stick-r"}, {RETRO_GAMEPAD_BUTTON_TYPE_HOME, "RETRO_GAMEPAD_BUTTON_TYPE_HOME", "home"}, {0, NULL, NULL}};
		GType retro_gamepad_button_type_type_id;
		retro_gamepad_button_type_type_id = g_enum_register_static ("RetroGamepadButtonType", values);
		g_once_init_leave (&retro_gamepad_button_type_type_id__volatile, retro_gamepad_button_type_type_id);
	}
	return retro_gamepad_button_type_type_id__volatile;
}



