/* retro-core-descriptor-error.c generated by valac 0.36.4, the Vala compiler
 * generated from retro-core-descriptor-error.vala, do not modify */

/* This file is part of retro-gtk. License: GPLv3*/

#include <glib.h>
#include <glib-object.h>


typedef enum  {
	RETRO_CORE_DESCRIPTOR_ERROR_REQUIRED_GROUP_NOT_FOUND,
	RETRO_CORE_DESCRIPTOR_ERROR_REQUIRED_KEY_NOT_FOUND,
	RETRO_CORE_DESCRIPTOR_ERROR_FIRMWARE_NOT_FOUND
} RetroCoreDescriptorError;
#define RETRO_CORE_DESCRIPTOR_ERROR retro_core_descriptor_error_quark ()


GQuark retro_core_descriptor_error_quark (void);


GQuark retro_core_descriptor_error_quark (void) {
	return g_quark_from_static_string ("retro_core_descriptor_error-quark");
}



