/* video.c generated by valac 0.36.4, the Vala compiler
 * generated from video.vala, do not modify */

/* This file is part of retro-gtk. License: GPLv3*/

#include <glib.h>
#include <glib-object.h>


#define RETRO_TYPE_PIXEL_FORMAT (retro_pixel_format_get_type ())

typedef enum  {
	RETRO_PIXEL_FORMAT_XRGB1555,
	RETRO_PIXEL_FORMAT_XRGB8888,
	RETRO_PIXEL_FORMAT_RGB565,
	RETRO_PIXEL_FORMAT_UNKNOWN = -1
} RetroPixelFormat;



GType retro_pixel_format_get_type (void) G_GNUC_CONST;


/**
 * Pixel formats used by Libretro.
 */
GType retro_pixel_format_get_type (void) {
	static volatile gsize retro_pixel_format_type_id__volatile = 0;
	if (g_once_init_enter (&retro_pixel_format_type_id__volatile)) {
		static const GEnumValue values[] = {{RETRO_PIXEL_FORMAT_XRGB1555, "RETRO_PIXEL_FORMAT_XRGB1555", "xrgb1555"}, {RETRO_PIXEL_FORMAT_XRGB8888, "RETRO_PIXEL_FORMAT_XRGB8888", "xrgb8888"}, {RETRO_PIXEL_FORMAT_RGB565, "RETRO_PIXEL_FORMAT_RGB565", "rgb565"}, {RETRO_PIXEL_FORMAT_UNKNOWN, "RETRO_PIXEL_FORMAT_UNKNOWN", "unknown"}, {0, NULL, NULL}};
		GType retro_pixel_format_type_id;
		retro_pixel_format_type_id = g_enum_register_static ("RetroPixelFormat", values);
		g_once_init_leave (&retro_pixel_format_type_id__volatile, retro_pixel_format_type_id);
	}
	return retro_pixel_format_type_id__volatile;
}



